//
// by Khlytin Grigoriy, 06.11.2022
// last update: 04.12.2022, 02:26 !
//

#include "testlib.h"

using namespace std;

typedef long long int ll;

// COMMENT: generates rectangle sides within valid ranges
pair<ll, ll> gen_a_b(ll MAX_A, ll MAX_B) {
    ll a = rnd.next(1LL, MAX_A);
    ll b = rnd.next(1LL, MAX_B);
    return {a, b};
}

// ===== TWO ROOTS =====
// x < a
// y < b
// x + y = k
// (x + 1) * (y + 1) = m

// COMMENT: valid roots
// must:
// (a + b) <= MAX_K + 2
// (a * b) <= MAX_M
void p_two_roots_certified_build(ll a, ll b, bool is_special) {
    ll x, y;
    if (!is_special) {
        ll r0 = rnd.next(0LL, 100000LL);
        ll r1 = rnd.next(0LL, 100000LL);
        if (r0 % 2LL == 0LL) {
            if (r1 % 3LL == 0LL) {
                x = rnd.next(0LL, min(10LL, a - 1LL));
                y = rnd.next(max(0LL, b - 10LL), b - 1LL);
            } else if (r1 % 3LL == 1LL) {
                x = rnd.next(max(0LL, a - 10LL), a - 1LL);
                y = rnd.next(0LL, min(10LL, b - 1LL));
            } else {
                x = rnd.next(max(0LL, a - 10LL), a - 1LL);
                y = rnd.next(max(0LL, b - 10LL), b - 1LL);
            }
        } else {
            x = rnd.next(0LL, a - 1LL);
            y = rnd.next(0LL, b - 1LL);
        }
    } else {
        if (a < b) {
            x = rnd.next(0LL, a - 1LL);
            y = rnd.next(a, b - 1LL);
        } else if (b < a) {
            x = rnd.next(b, a - 1LL);
            y = rnd.next(0LL, b - 1LL);
        } else {
            x = rnd.next(0LL, a - 1LL);
            y = rnd.next(0LL, b - 1LL);
        }
    }
    ll k = x + y;
    ll m = (x + 1LL) * (y + 1LL);
    println(a, b, k, m);
}

// COMMENT: invalid roots
// must:
// a <= MAX_X
// b <= MAX_Y
// (MAX_X + MAX_Y) <= MAX_K
// (MAX_X + 1) * (MAX_Y + 1) <= MAX_M
void p_two_roots_invalid_build(ll a, ll b, ll MAX_X, ll MAX_Y) {
    ll x, y;
    x = rnd.next(a, MAX_X);
    y = rnd.next(b, MAX_Y);
    ll k = x + y;
    ll m = (x + 1LL) * (y + 1LL);
    println(a, b, k, m);
}

// COMMENT: roots of all types
// COMMENT: generates the number of horizontal and vertical cuts
// COMMENT: 1) roots are less than the sides
// COMMENT: 2) one root is greater than the side of the rectangle
// COMMENT: 3) roots are greater than the sides
// must:
// -----
// (a + b) <= MAX_K + 2
// (a * b) <= MAX_M
// -----
// a <= MAX_X
// b <= MAX_Y
// (MAX_X + MAX_Y) <= MAX_K
// (MAX_X + 1) * (MAX_Y + 1) <= MAX_M
// -----
void two_roots_build(ll a, ll b, ll MAX_X, ll MAX_Y) {
    ll r = rnd.next(0LL, 100000LL);
    if (r % 3LL == 0LL) {
        p_two_roots_certified_build(a, b, false);
    } else if (r % 3LL == 1LL) {
        p_two_roots_certified_build(a, b, true);
    } else {
        p_two_roots_invalid_build(a, b, MAX_X, MAX_Y);
    }
}

// ===== ONE ROOT =====
// (-1) * x^2 + (k) * x + (k - m + 1) = 0
// D = k^2 - 4 * (-1) * (k - m + 1) = 0
// k^2 + 4k + 4 = 4m
// (k + 2)^2 = 4m
// m = (k + 2)^2 / 4
// x = (k / 2) < a
// y = (k / 2) < b

// COMMENT: valid roots
// must:
// MAX_K <= 2 * sqrt(MAX_M) - 2
// make:
// MAX_K <= min(a * 2, b * 2) - 1
void p_one_root_certified_build(ll a, ll b, ll MAX_K) {
    MAX_K = min(MAX_K, min(a * 2LL, b * 2LL) - 1LL);
    ll k = rnd.next(0LL, MAX_K / 2LL) * 2LL;
    ll m = (k + 2LL) * (k + 2LL) / 4LL;
    println(a, b, k, m);
}

// COMMENT: invalid roots
// must:
// MAX_K <= 2 * sqrt(MAX_M) - 2
void p_one_root_invalid_build(ll a, ll b, ll MAX_K) {
    ll k = rnd.next(MAX_K / 4LL, MAX_K / 2LL) * 2LL;
    ll m = (k + 2LL) * (k + 2LL) / 4LL;
    println(a, b, k, m);
}

// COMMENT: roots of all types
// COMMENT: generates a quadratic equation with zero discriminant
// COMMENT: both roots are equal to each other
// COMMENT: 1) roots are less than the sides
// COMMENT: 2) roots can be greater than sides
// must:
// -----
// MAX_K <= 2 * sqrt(MAX_M) - 2
// -----
void one_root_build(ll a, ll b, ll MAX_K) {
    ll r = rnd.next(0LL, 100000LL);
    if (r % 2LL == 0LL) {
        p_one_root_certified_build(a, b, MAX_K);
    } else {
        p_one_root_invalid_build(a, b, MAX_K);
    }
}

// ===== RANDOM =====

void p_rand_build_1(ll a, ll b, ll MAX_K, ll MAX_M) {
    ll k = rnd.next(0LL, MAX_K);
    ll m = rnd.next(k + 1LL, MAX_M);
    println(a, b, k, m);
}

void p_rand_build_2(ll a, ll b, ll MAX_K) {
    ll k = rnd.next(0LL, MAX_K);
    ll m = k + 1LL;
    println(a, b, k, m);
}

void p_rand_build_3(ll a, ll b, ll MAX_K, ll MAX_M) {
    ll k = rnd.next(0LL, MAX_K);
    ll m = rnd.next(k + 1LL, min(2LL * MAX_K, MAX_M));
    println(a, b, k, m);
}

// COMMENT: generates random "k" and "m" (and includes the case "m = k + 1")
// must:
// -----
// MAX_K < MAX_M
// -----
void rand_build(ll a, ll b, ll MAX_K, ll MAX_M) {
    ll r = rnd.next(0LL, 100000LL);
    if (r % 3LL == 0LL) {
        p_rand_build_1(a, b, MAX_K, MAX_M);
    } else if (r % 3LL == 1LL) {
        p_rand_build_2(a, b, MAX_K);
    } else {
        p_rand_build_3(a, b, MAX_K, MAX_M);
    }
}

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);

    ll TYPE_TEST = opt<ll>("type");
    ll GROUP = opt<ll>("group");
    ll MAX_VAL = opt<ll>("maxval");

    //

    ll TESTS_COUNT = 100LL;

    ll GLOBAL_MAX_A = (ll) 1e9;
    ll GLOBAL_MAX_B = (ll) 1e9;
    ll GLOBAL_MAX_K = 2LL * (ll) 1e9;
    ll GLOBAL_MAX_M = (ll) 1e18;

    // invariant: GLOBAL_MAX_K < GLOBAL_MAX_M

    if (MAX_VAL >= 3LL) {
        GLOBAL_MAX_A = MAX_VAL;
        GLOBAL_MAX_B = MAX_VAL;
        GLOBAL_MAX_K = 2LL * MAX_VAL;
        GLOBAL_MAX_M = MAX_VAL * MAX_VAL;
    }

    if (GROUP == 1LL) {
        GLOBAL_MAX_A = 1LL;
    } else if (GROUP == 2LL) {
        GLOBAL_MAX_M = min((ll) 1e5, GLOBAL_MAX_M);
        GLOBAL_MAX_K = min((ll) 1e5 - 1LL, GLOBAL_MAX_K);
    } else if (GROUP == 3LL) {
        GLOBAL_MAX_K = min((ll) 1e5, GLOBAL_MAX_K);
    } else if (GROUP == 4LL) {
        GLOBAL_MAX_M = min((ll) 1e9, GLOBAL_MAX_M);
        GLOBAL_MAX_K = min((ll) 1e9 - 1LL, GLOBAL_MAX_K);
    }

    //

    ll val_a = GLOBAL_MAX_A;
    ll val_b = GLOBAL_MAX_B;
    ll val_k = GLOBAL_MAX_K;
    ll val_m = GLOBAL_MAX_M;

    // ----- TYPE_TEST : 0 ----- (small bruteforce)

    if (TYPE_TEST == 0) {
        ll cnt = 0;
        ll side_a = min(val_a, 3LL);
        ll side_b = min(val_b, 3LL);

        for (ll a = 1LL; a <= side_a; a++) {
            for (ll b = 1LL; b <= side_b; b++) {
                for (ll k = 0LL; k <= 4LL; k++) {
                    for (ll m = k + 1LL; m <= k + 2LL; m++) {
                        cnt++;
                    }
                }
            }
        }

        println(cnt);

        for (ll a = 1LL; a <= side_a; a++) {
            for (ll b = 1LL; b <= side_b; b++) {
                for (ll k = 0LL; k <= 4LL; k++) {
                    for (ll m = k + 1LL; m <= k + 2LL; m++) {
                        println(a, b, k, m);
                    }
                }
            }
        }

        return 0;
    }

    // ----- TYPE_TEST : [1, 3] ----- (bruteforce: a <= 10, b <= 10)

    if (TYPE_TEST >= 1LL && TYPE_TEST <= 3LL) {
        ll side_a = min(val_a, 10LL);
        ll side_b = 100LL / side_a;
        ll t = side_a * side_b;
        println(t);
        for (ll a = 1LL; a <= side_a; a++) {
            for (ll b = 1LL; b <= side_b; b++) {

                if (TYPE_TEST == 1LL) {
                    ll tmp_limit = min((ll) sqrt(val_m) - 1LL, val_k / 2LL);
                    // must:
                    // -----
                    // (a + b) <= MAX_K + 2
                    // (a * b) <= MAX_M
                    // -----
                    // a <= MAX_X
                    // b <= MAX_Y
                    // (MAX_X + MAX_Y) <= MAX_K
                    // (MAX_X + 1) * (MAX_Y + 1) <= MAX_M
                    // -----
                    two_roots_build(
                            min(tmp_limit, a),
                            min(tmp_limit, b),
                            tmp_limit,
                            tmp_limit
                    );
                } else if (TYPE_TEST == 2LL) {
                    ll tmp_limit = 2LL * (ll) sqrt(val_m) - 2LL;
                    // must:
                    // -----
                    // MAX_K <= 2 * sqrt(MAX_M) - 2
                    // -----
                    one_root_build(
                            a,
                            b,
                            min(tmp_limit, val_k)
                    );
                } else {
                    // must:
                    // -----
                    // MAX_K < MAX_M
                    // -----
                    rand_build(
                            a,
                            b,
                            val_k,
                            val_m
                    );
                }

            }
        }

        return 0;
    }

    // ----- TYPE_TEST : default ----- (3t launches)

    ll t = rnd.next(max(1LL, TESTS_COUNT / 4LL), max(1LL, TESTS_COUNT / 3LL));
    println(3LL * t);

    for (ll i = 0LL; i < t; i++) {
        pair<ll, ll> vars_a_b;

        ll tmp_limit_1 = min((ll) sqrt(val_m) - 1LL, val_k / 2LL);
        // must:
        // -----
        // (a + b) <= MAX_K + 2
        // (a * b) <= MAX_M
        // -----
        // a <= MAX_X
        // b <= MAX_Y
        // (MAX_X + MAX_Y) <= MAX_K
        // (MAX_X + 1) * (MAX_Y + 1) <= MAX_M
        // -----
        vars_a_b = gen_a_b(min(tmp_limit_1, val_a), min(tmp_limit_1, val_b));
        two_roots_build(
                vars_a_b.first,
                vars_a_b.second,
                tmp_limit_1,
                tmp_limit_1
        );

        ll tmp_limit_2 = 2LL * (ll) sqrt(val_m) - 2LL;
        // must:
        // -----
        // MAX_K <= 2 * sqrt(MAX_M) - 2
        // -----
        vars_a_b = gen_a_b(val_a, val_b);
        one_root_build(
                vars_a_b.first,
                vars_a_b.second,
                min(tmp_limit_2, val_k)
        );

        // must:
        // -----
        // MAX_K < MAX_M
        // -----
        vars_a_b = gen_a_b(val_a, val_b);
        rand_build(
                vars_a_b.first,
                vars_a_b.second,
                val_k,
                val_m
        );
    }

    return 0;
}
