#include <iostream>
#include <random>
#include <thread>
#include <vector>
#include <iomanip>
#include <set>
#include <map>
#include <algorithm>

using namespace std;

#define int long long

const int MAXN = 8e5 + 5;
long long p[MAXN];
pair<int, long long> t[MAXN];
int inc[MAXN];

long long comp_delta[MAXN];

pair<int, long long> merge(pair<int, long long> a, pair<int, long long> b) {
    if (b.second == 0) {
        return a;
    }
    if (a.second == 0) {
        return b;
    }
    if (a.first > b.first) {
        swap(a, b);
    }
    if (a.first == b.first) {
        return {a.first, a.second + b.second};
    }
    return a;
}

void build(int v, int l, int r) {
    if (r - l <= 0) {
        return;
    }
    if (r - l == 1) {
        p[v] = comp_delta[l];
        t[v] = {0, p[v]};
        inc[v] = 0;
        return;
    }
    int m = (l + r) / 2;
    build(v * 2, l, m);
    build(v * 2 + 1, m, r);
    t[v] = merge(t[v * 2], t[v * 2 + 1]);
    p[v] = p[v * 2] + p[v * 2 + 1];
    inc[v] = 0;
}

void push(int v, int l, int r) {
    if (inc[v] != 0) {
        t[v].first += inc[v];
        inc[v * 2] += inc[v];
        inc[v * 2 + 1] += inc[v];
        inc[v] = 0;
    }
}

void add(int v, int l, int r, int ql, int qr, int delta) {
    if (r - l <= 0) {
        return;
    }
    push(v, l, r);
    if (ql <= l && r <= qr) {
        inc[v] += delta;
        push(v, l, r);
        return;
    }
    if (qr <= l || r <= ql) {
        return;
    }
    int m = (r + l) / 2;
    add(v * 2, l, m, ql, qr, delta);
    add(v * 2 + 1, m, r, ql, qr, delta);
    t[v] = merge(t[v * 2 + 1], t[v * 2]);
}


long long zero_cnt() {
    if (t[1].first == 0) {
        return t[1].second;
    }
    return 0;
}

struct border {
    long long y;
    long long x1, x2;
    int delta;

    bool operator<(const border &rhs) const {
        if (y < rhs.y)
            return true;
        if (rhs.y < y)
            return false;
        if (x1 < rhs.x1)
            return true;
        if (rhs.x1 < x1)
            return false;
        if (x2 < rhs.x2)
            return true;
        if (rhs.x2 < x2)
            return false;
        return delta < rhs.delta;
    }

    bool operator>(const border &rhs) const {
        return rhs < *this;
    }

    bool operator<=(const border &rhs) const {
        return !(rhs < *this);
    }

    bool operator>=(const border &rhs) const {
        return !(*this < rhs);
    }
};

vector<long long> xs;
vector<border> bs;

void solve() {
    sort(xs.begin(), xs.end());
    for (int i = 0; i < xs.size(); ++i) {
        comp_delta[i] = xs[i + 1] - xs[i];
    }
    comp_delta[xs.size() - 1] = 0;

    map<long long, int> mp;
    for (int i = 0; i < xs.size(); ++i) {
        mp[xs[i]] = i;
    }

    build(1, 0, xs.size());
    sort(bs.begin(), bs.end());
    long long start = zero_cnt();
    long long ans = 0;
    long long ly = 0;
    for (auto e: bs) {
        ans += (e.y - ly) * (start - zero_cnt());
        ly = e.y;
        add(1, 0, xs.size(), mp[e.x1], mp[e.x2], e.delta);
    }
    cout << ans;
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
    int k, n;
    cin >> k >> n;
    long long x = 0;
    long long y = 0;
    for (int i = 0; i < n; ++i) {
        string d;
        long long a;
        cin >> d >> a;
        long long xn = x;
        long long yn = y;
        if (d == "N") {
            yn += a;
        } else if (d == "S") {
            yn -= a;
        } else if (d == "W") {
            xn -= a;
        } else if (d == "E") {
            xn += a;
        }

        long long ymx = max(y + k, yn + k);
        long long ymn = min(y, yn);
        long long xmx = max(x + k, xn + k);
        long long xmn = min(x, xn);

        xs.push_back(xmx);
        xs.push_back(xmn);
        bs.push_back(border{ymn, xmn, xmx, 1});
        bs.push_back(border{ymx, xmn, xmx, -1});
        x = xn;
        y = yn;
    }
    solve();
}

